
%*==============================================================================================*;
%* Program name:	macro_convert_vertical_tree 												*;														*;
%* Date created:	04/23/2015																	*;
%* Version:			1.0																			*;
%* Last modified:	04/23/2015																	*;
%*==============================================================================================*;
%* Program description:		Convert a vertical tree in child-parent format to dx-node format    *;
%*																								*;
%* Documentation: 			Mini-Sentinel PRISM Tech Spec - PRISM Data Mining Infrastructure 	*;
%*							alert capture														*;
%* Documentation version: 	1.0																	*;
%*																								*;
%* Dependencies:																				*;
%*		- requires input tree file in a two-column dataset with Child and Parent variables 		*;
%*		- the tree levels in the parent variable must be in a format using a common delimiter 	*;
%*		  such as "." to distinguish levels														*;
%*		- tree must have at least three levels, e.g. 11.22.33, to distinguish nodes from ICD-9	*;
%*		  codes																					*;
%*==============================================================================================*;
%* CHANGE LOG 																					*;
%*----------------------------------------------------------------------------------------------*;
%* Version		Date		Comments														*;
%* -------		----------	--------														*;
%* v1.0			04/23/2015	- first production version										*;
%*==============================================================================================*;

%* NOTE: This program is for local use only and not intended for distribution 					*;

%*----------------------------------------------------------------------------------------------*;
%* Macro to convert vertical tree file															*;
%*----------------------------------------------------------------------------------------------*;

%* Parameter instructions:																		*;
	%* Use two-level SAS name for datasaet name parameters										*;
	%* Delim must be the character by itself with no quotation marks							*;

%macro convert_vertical_tree(  ChildParentDSName	=
						  	 , DxNodeDSName			=
							 , LevelDlm				= 
							
						     );

	%* ---------- BEGIN CONVERT_VERTICAL_TREE MACRO ----------									*;

	* Determine number of levels																*;
	proc sql noprint;
		select max(countc(parent,"&LevelDlm.")) into: maxdelim
		from &ChildParentDSName.
		;
	quit;

	%let maxdelim=&maxdelim.;
	%put ----- Maximum delimiter count: &maxdelim. -----;

	%let maxlevel=%eval(&maxdelim. +2);
	%put ----- Number of levels: &maxlevel. -----;

	%* Create dx-node dataset from child-parent dataset											*;
	data &DxNodeDSName. (keep=dx node dx_codetype level);
		set &ChildParentDSName.;
		dx_codetype="&tree_dx_codetype.";
		where countc(parent,"&LevelDlm.") = &maxdelim.;
		dx = child;
		node = dx;
		level=&maxlevel.;
		output;
		dx = child;
		node = parent;
		level=&maxlevel.-1;
		output;
		%do i=&maxdelim. %to 1 %by -1;
			call scan(parent, &i., position, length, "&LevelDlm.");
			dx = child;
			node = substr(parent, 1, position + length -1);
			level=&i;
			output;
		%end;
	run;
       


	%* ---------- END CONVERT_VERTICAL_TREE MACRO ----------									*;

	
%mend convert_vertical_tree;
